#!/bin/bash

i=1				# Counter for position in list of domain names
filelength=`cat $1 | wc -l`	# Length of list of domain names

# Loop through file
while [ "$i" -le "$filelength" ] ; do

dig_item=`head -n $i $1 | tail -n 1`	# dig_item is the name being dug

echo "?" >> getnameservers.log
echo "$i" >> getnameservers.log
echo "$dig_item"
echo "$dig_item" >> getnameservers.log

# Check for an empty dig_item
if [ -z "$dig_item" ] ; then
 echo "Empty dig_item" >> getnameservers.log
 i=`expr $i + 1`
 continue
fi

# Check for a dig_item not containing . (means a common incorrect name)
 namecheck=`echo $dig_item | grep '\.'`
if [ -z "$namecheck" ] ; then
 echo "No . in dig_item" >> getnameservers.log
 i=`expr $i + 1`
 continue
fi

dig "$dig_item" NS > temp	# Perform dig and put in temp file

# Check to see if the dig returns a SERVFAIL status
 status=`cat temp | grep HEADER | cut -d ':' -f 3 | cut -d ',' -f 1 | tr -d ' '`
if [ "$status" = "SERVFAIL" ] ; then
 echo "SERVFAIL" >> getnameservers.log
 i=`expr $i + 1`
 continue
fi

# Get values of number of entries in each section of query response
question=`cat temp | grep flags | cut -d ':' -f 3 | cut -d ',' -f 1`
answer=`cat temp | grep flags | cut -d ':' -f 4 | cut -d ',' -f 1`
authority=`cat temp | grep flags | cut -d ':' -f 5 | cut -d ',' -f 1`
additional=`cat temp | grep flags | cut -d ':' -f 6 | cut -d ',' -f 1`

# Check for SOA records, add new domain name to search to bottom of sample
if [ "$answer" -eq 0 ] ; then
 echo "SOA found/No answers" >> getnameservers.log
 echo "$dig_item" | cut -d '.' -f 2- >> "$1"
 filelength=`expr $filelength + 1`
 i=`expr $i + 1`
 continue
fi


# Get nameserver names
nameserver=`cat temp | grep -A "$answer" "ANSWER SECTION" | grep -v "ANSWER SECTION" | grep NS | tr '\t' ' ' | tr -s ' ' | cut -d ' ' -f 5`
echo "$nameserver"
if [ -n "$nameserver" ] ; then
 echo "Getting nameservers..." >> getnameservers.log
 for nameserver_name in `echo "$nameserver"` ; do
  echo "$nameserver_name"
  echo "$nameserver_name" >> getnameservers.log

  ip=`cat temp | grep -A "$additional" "ADDITIONAL SECTION" | grep "$nameserver_name" | grep -v "AAAA" | tr '\t' ' ' | tr -s ' ' | cut -d ' ' -f 5`
  if [ -z "$ip" ] ; then
   ip=`dig "$nameserver_name" A | grep -A 1 "ANSWER SECTION" | grep -v "ANSWER SECTION" | grep -v "AAAA" | tr '\t' ' ' | tr -s ' ' | cut -d ' ' -f 5`
  fi

  for ip_address in `echo "$ip"` ; do
   echo -n "$dig_item" >> nameservers
   echo -n "?" >> nameservers
   echo -n "$nameserver_name" >> nameservers
   echo -n "?" >> nameservers

   echo "$ip_address"
   echo "$ip_address" >> getnameservers.log
   echo "$ip_address" >> nameservers

  done

 done
 i=`expr $i + 1`
 continue
fi

# Check for CNAME
 tmp_string=`cat temp | grep CNAME`
if [ -n tmp_string ] ; then
 cname=`echo "$tmp_string" | tr '\t' ' ' | tr -s ' ' | cut -d ' ' -f 5`
 echo -n "CNAME found: " >> getnameservers.log
 echo "$cname" >> getnameservers.log
 echo "$cname" >> "$1"
 filelength=`expr $filelength + 1`
 i=`expr $i + 1`
 continue
fi

echo -n "Error: "
echo -n "$dig_item"
echo " not dug correctly!"

echo -n "Error: " >> getnameservers.log
echo -n "$dig_item" >> getnameservers.log
echo " not dug correctly!" >> getnameservers.log

done

rm temp
